@@ -17,3 +17,10 @@ reproduction_minimum = lambda: 10
 movement_energy = lambda: 0.25
 growth_minimum = lambda: 10
 energy_need = lambda: 0.2
+algae_limit = lambda: 20
+newborn_limit = lambda: 9
+reproduction_probability = lambda: 0.8
+growth_probability = lambda: 0.8
+growth_cost_factor = lambda: 0.5
+capacity_factor = lambda: 1.1
+initial_algae_probability = lambda: 0.3
