@@ -3,6 +3,7 @@ from random import random, sample
 
 from pyage.core.address import Addressable
 from pyage.core.inject import Inject
+from pyage_forams.solutions.agent.aggregate import ForamAggregateAgent
 from pyage_forams.solutions.genom import Genom
 from pyage_forams.utils import counted
 
@@ -10,30 +11,6 @@ from pyage_forams.utils import counted
 logger = logging.getLogger(__name__)
 
 
-class ForamAggregateAgent(Addressable):
-    @Inject("forams", "environment")
-    def __init__(self):
-        super(ForamAggregateAgent, self).__init__()
-        for foram in self.forams.values():
-            foram.parent = self
-            self.environment.add_foram(foram)
-
-    def step(self):
-        for foram in self.forams.values():
-            foram.step()
-        self.environment.tick(self.parent.steps)
-
-    def remove_foram(self, address):
-        foram = self.forams[address]
-        del self.forams[address]
-        foram.parent = None
-        return foram
-
-    def add_foram(self, foram):
-        foram.parent = self
-        self.forams[foram.get_address()] = foram
-
-
 class Foram(Addressable):
     @Inject("genom_factory", "reproduction_minimum", "movement_energy", "growth_minimum", "energy_need",
             "newborn_limit", "reproduction_probability", "growth_probability", "growth_cost_factor", "capacity_factor")
@@ -44,16 +21,15 @@ class Foram(Addressable):
             self.genom = self.genom_factory()
         else:
             self.genom = genom
-        self.steps = 0
         self.chambers = 1
         self.alive = True
         self.cell = None
 
     def step(self):
+        logger.debug("step %s", self)
         if not self.alive:
             logger.warn("called step on dead foram")
             return
-        self.steps += 1
         if self._should_die():
             self._die()
             return
@@ -79,7 +55,7 @@ class Foram(Addressable):
 
     def _take_algae(self, capacity):
         taken = 0
-        cells = iter([self.cell] + self.cell.neighbours)
+        cells = iter([self.cell] + self.cell.get_neighbours())
         while capacity > taken:
             try:
                 cell = cells.next()
@@ -94,7 +70,8 @@ class Foram(Addressable):
 
     @counted
     def _reproduce(self):
-        empty_neighbours = filter(lambda c: c.is_empty(), self.cell.neighbours)
+        logger.debug("%s is reproducing" % self)
+        empty_neighbours = filter(lambda c: c.is_empty(), self.cell.get_neighbours())
         if not empty_neighbours:
             logger.debug("%s has no space to reproduce" % self)
             return
@@ -111,18 +88,19 @@ class Foram(Addressable):
     @counted
     def _create_child(self, cell, energy):
         foram = Foram(energy, Genom(self.genom.chambers_limit))
-        cell.insert_foram(foram)
         self.parent.add_foram(foram)
+        cell.insert_foram(foram)
 
     def _move(self):
         try:
-            empty_neighbours = filter(lambda c: c.is_empty(), self.cell.neighbours)
+            empty_neighbours = filter(lambda c: c.is_empty(), self.cell.get_neighbours())
             if not empty_neighbours:
                 logger.warning("%s has nowhere to move" % self)
                 return
             logger.debug(empty_neighbours)
             cell = max(empty_neighbours,
-                       key=lambda c: random() + c.available_food() + sum(s.available_food() for s in c.neighbours))
+                       key=lambda c: random() + c.available_food() + sum(
+                           s.available_food() for s in c.get_neighbours()))
             if cell:
                 cell.insert_foram(self.cell.remove_foram())
                 self.energy -= self.movement_energy
@@ -150,7 +128,7 @@ class Foram(Addressable):
         logger.debug("Foram %s has a new chamber, so %d altogether" % (self, self.chambers))
 
     def __repr__(self):
-        return "Foram[%.2f, %d]" % (self.energy, self.chambers)
+        return "%s[%.2f, %d]" % (self.get_address(), self.energy, self.chambers)
 
 
 def create_forams(count, initial_energy):
